@@ -1,5 +1,5 @@
 use crate::builder::{Builder, RunConfig, ShouldRun, Step};
-use build_helper::t;
+use crate::util::t;
 use serde::{Deserialize, Serialize};
 use std::collections::HashMap;
 use std::env;
@@ -24,7 +24,7 @@ const OS: Option<&str> = None;
 
 type ToolstateData = HashMap<Box<str>, ToolState>;
 
-#[derive(Copy, Clone, Debug, Deserialize, Serialize, PartialEq, Eq, PartialOrd)]
+#[derive(Copy, Clone, Debug, Deserialize, Serialize, PartialEq, PartialOrd)]
 #[serde(rename_all = "kebab-case")]
 /// Whether a tool can be compiled, tested or neither
 pub enum ToolState {
@@ -50,13 +50,6 @@ impl fmt::Display for ToolState {
     }
 }
 
-impl Default for ToolState {
-    fn default() -> Self {
-        // err on the safe side
-        ToolState::BuildFail
-    }
-}
-
 /// Number of days after the last promotion of beta.
 /// Its value is 41 on the Tuesday where "Promote master to beta (T-2)" happens.
 /// The Wednesday after this has value 0.
@@ -77,7 +70,6 @@ static STABLE_TOOLS: &[(&str, &str)] = &[
     ("rust-by-example", "src/doc/rust-by-example"),
     ("edition-guide", "src/doc/edition-guide"),
     ("rls", "src/tools/rls"),
-    ("rustfmt", "src/tools/rustfmt"),
 ];
 
 // These tools are permitted to not build on the beta/stable channels.
@@ -152,7 +144,7 @@ impl Step for ToolStateCheck {
     /// error if there are any.
     ///
     /// This also handles publishing the results to the `history` directory of
-    /// the toolstate repo https://github.com/rust-lang-nursery/rust-toolstate
+    /// the toolstate repo <https://github.com/rust-lang-nursery/rust-toolstate>
     /// if the env var `TOOLSTATE_PUBLISH` is set. Note that there is a
     /// *separate* step of updating the `latest.json` file and creating GitHub
     /// issues and comments in `src/ci/publish_toolstate.sh`, which is only
@@ -162,7 +154,7 @@ impl Step for ToolStateCheck {
     /// The rules for failure are:
     /// * If the PR modifies a tool, the status must be test-pass.
     ///   NOTE: There is intent to change this, see
-    ///   https://github.com/rust-lang/rust/issues/65000.
+    ///   <https://github.com/rust-lang/rust/issues/65000>.
     /// * All "stable" tools must be test-pass on the stable or beta branches.
     /// * During beta promotion week, a PR is not allowed to "regress" a
     ///   stable tool. That is, the status is not allowed to get worse
@@ -278,10 +270,9 @@ impl Builder<'_> {
         if self.config.dry_run {
             return;
         }
-        // Toolstate isn't tracked for clippy, but since most tools do, we avoid
-        // checking in all the places we could save toolstate and just do so
-        // here.
-        if tool == "clippy-driver" {
+        // Toolstate isn't tracked for clippy or rustfmt, but since most tools do, we avoid checking
+        // in all the places we could save toolstate and just do so here.
+        if tool == "clippy-driver" || tool == "rustfmt" {
             return;
         }
         if let Some(ref path) = self.config.save_toolstates {
@@ -468,13 +459,11 @@ fn publish_test_results(current_toolstate: &ToolstateData) {
     t!(fs::write(&history_path, file));
 }
 
-#[derive(Debug, Serialize, Deserialize)]
+#[derive(Debug, Deserialize)]
 struct RepoState {
     tool: String,
     windows: ToolState,
     linux: ToolState,
-    commit: String,
-    datetime: String,
 }
 
 impl RepoState {
