@@ -1,5 +1,3 @@
-#![feature(const_transmute)]
-
 // normalize-stderr-64bit "64 bits" -> "word size"
 // normalize-stderr-32bit "32 bits" -> "word size"
 // normalize-stderr-64bit "128 bits" -> "2 * word size"
@@ -15,6 +13,7 @@ fn main() {
 const ZST: &[u8] = unsafe { std::mem::transmute(1usize) };
 //~^ ERROR any use of this value will cause an error
 //~| ERROR cannot transmute between types of different sizes
+//~| WARN this was previously accepted by the compiler but is being phased out
 
 // Once the `any use of this value will cause an error` disappears in this test, make sure to
 // remove the `TransmuteSizeDiff` error variant and make its emitter site an assertion again.
