@@ -1,9 +1,15 @@
-// only-aarch64
 // build-fail
+// compile-flags: --crate-type=rlib --target=aarch64-unknown-linux-gnu
+// needs-llvm-components: aarch64
+#![cfg_attr(bootstrap, feature(aarch64_target_feature))]
+#![feature(no_core, lang_items)]
+#![no_core]
 
-#![feature(aarch64_target_feature, target_feature_11)]
+#[lang="sized"]
+trait Sized {}
 
-fn main() {
+// FIXME: this should not need to be public.
+pub fn main() {
     #[target_feature(enable = "pacg")]
     //~^ ERROR must all be either enabled or disabled together
     unsafe fn inner() {}
