@@ -6,11 +6,7 @@ fn main() {}
 
 // test that unused generic parameters are ok
 type Two<T, U> = impl Debug;
-
-fn one<T: Debug>(t: T) -> Two<T, T> {
-    //~^ ERROR non-defining opaque type use in defining scope
-    t
-}
+//~^ ERROR `T` doesn't implement `Debug`
 
 fn two<T: Debug, U>(t: T, _: U) -> Two<T, U> {
     t
@@ -18,4 +14,5 @@ fn two<T: Debug, U>(t: T, _: U) -> Two<T, U> {
 
 fn three<T, U: Debug>(_: T, u: U) -> Two<T, U> {
     u
+    //~^ ERROR concrete type differs from previous defining opaque type use
 }
