@@ -15,13 +15,13 @@ struct X;
 
 impl Foo for X {
     type Bar = impl Baz<Self, Self>;
-    //~^ ERROR mismatched types
 
     fn bar(&self) -> Self::Bar {
         |x| x
+        //~^ ERROR implementation of `FnOnce` is not general enough
     }
 }
 
-trait Baz<A, B> = Fn(&A) -> &B;
+trait Baz<A: ?Sized, B: ?Sized> = Fn(&A) -> &B;
 
 fn main() {}
